#
# documentation:
import requests
import json
import urllib3

# Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

#Disable warnings https
urllib3.disable_warnings()

class WebAppHealthChecks:
    def __init__(self, url, app_name):
        
        #initial
        self.url = url
        self.app_name = app_name
               

    def get_status_data(self):
            
        #Add tags to the URL
        url = self.url
        app_name = self.app_name
       
        # requests doc http://docs.python-requests.org/en/v0.10.7/user/quickstart/#custom-headers
        r = requests.get(url=url, verify=False)
        
        return r.json(), r.status_code
    
    def check_status_data(self):

        #Vars
        retrcode = OK
		#Create tuple with json and status code
        webapp_health_status = self.get_status_data()       

        msgdata = ''
        msgerror = '{:>10}'.format(webapp_health_status[0].get('entries').get(self.app_name).get('description', ''))           
        retrperfdata = ''
        retrmsg = ''
        
        #Validate Data
        if webapp_health_status[0]['status'] != 'Healthy':
            retrcode = CRITICAL

        msgerror += msgdata
         
        return retrcode, msgerror
        


#!python3
# https://nagios-plugins.org/doc/guidelines.html

# Import required libs
from .plugin_check import curlCheck
from .webapp_health_check import WebAppHealthChecks
import argparse
import sys


# Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

# Return message
message = {
    'status': OK,
    'summary': 'Example summary',
    'perfdata': 'label1=0;;;; '  # 'label'=value[UOM];[warn];[crit];[min];[max] 
}

# For multiple perdata, ensure to add space after each perfdata
# message['perfdata'] = 'label1=x;;;; '
# message['perfdata'] += 'label2=x;;;; '

# Function to parse arguments
def parse_args(args):
    """
    Information extracted from: https://mkaz.com/2014/07/26/python-argparse-cookbook/
     https://docs.python.org/3/library/argparse.html
    :return: parse.parse_args(args) object
    You can use obj.option, example:
    options = parse_args(args)
    options.user # to read username
    """
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, 
                                     description='nagios plugin to check some url using curl and some other code')

    parser.add_argument('-u', '--url', dest='url', nargs='?', default=None, const=None,
                        help='url to check \n')              
    parser.add_argument('-a', '--app_name', dest='app_name', nargs='?', default=None, const=None,
                        help='app_name is the key entry where the description is located\n')     
    parser.add_argument('-e', '--extra_args', dest='extra_args', nargs='?', default='', const=None,
                            help='extra args to add to curl, see curl manpage  \n')
    
    if not args:
        raise SystemExit(parser.print_help())

    return parser.parse_args(args)

# Function to execute cli commands
def cli_execution(options):
    """
    : param: options: arguments from parse.parse_args(args) (see parse_args function)
    """
    
    #Create web app health object    
    webapp_healt_obj = WebAppHealthChecks(url=options.url, app_name= options.app_name)

    def collect_data():        
        retrcode, msgdata = webapp_healt_obj.check_status_data()
        return retrcode, msgdata
    
    def check(retrcode):
        if retrcode >= 2:
            status = CRITICAL
            message['summary'] = 'CRITICAL: '            
        elif retrcode == 1:
            status = WARNING
            message['summary'] = 'WARNING: '
        else:
            status = OK
            message['summary'] = 'OK: '
        return status
        
   
    # Check logic starts here
    data = collect_data()
    message['status'] = check(data[0])
    # Add summary       
    message['summary'] += data[1]
    # Add perfdata
  
    total = len(data)
    message['perfdata'] = "total_time={};1;0;0; ".format(total)      

    # Print the message    
    print("{summary}|{perfdata}".format(
        summary=message.get('summary'),
        perfdata=message.get('perfdata')
    ))
    # Exit with status code
    raise SystemExit(message['status'])

# Argument parser
# https://docs.python.org/3.5/library/argparse.html

def main():
    """
    Main function
    """
    # Get options with argparse
    options = parse_args(sys.argv[1:])
    # Execute program functions passing the options collected
    cli_execution(options)


if __name__ == "__main__":
    main()



# plugin_check.py file with classes or functions to collect data or format messages
# This is just an example class from healthMonitor
import json
import subprocess

class curlCheck:
    def __init__(self, URL, app_name, extra_args='', auth_args=''):
        """
        :param URL: provide url to get/put data.
        :param extra_args: any argument for curl command
        :param auth_args: normally used internally by the code to add HEADERS with authentication args.
        """
        self.url = URL
        self.app_name = app_name
        self.extra_args = extra_args
        self.auth_args = auth_args
        
        self.std_args = '''" {\\"response_code\\": \\"%{http_code}\\",
        \\"dns_time\\": \\"%{time_namelookup}\\",
        \\"connect_time\\": \\"%{time_connect}\\",
        \\"pretransfer_time\\": \\"%{time_pretransfer}\\",
        \\"starttransfer_time\\": \\"%{time_starttransfer}\\",
        \\"total_time\\": \\"%{time_total}\\"
        "} '''

    def collect_data(self):
        """

        return: tuple
        returncode, return output, dict output with info ex:
        {'pretransfer_time': '0.000', 
        'response_code': '200', 
        'starttransfer_time': '0.182', 
        'dns_time': '0.000', 
        'connect_time': '0.000', 
        'total_time': '0.249'}
        """
        ### some code here
        ### or calling external modules
        cmdline = "curl {} -k --fail -s -o /dev/null {} -w \\ {}  {}".format(self.url, self.app_name, self.extra_args, self.std_args, self.auth_args)

        retrcode, retroutput = subprocess.getstatusoutput(cmdline)  

        jsonoutput = json.loads(retroutput)

        return retrcode, retroutput, jsonoutput
    
    def format_perfdata(self):

        perfdata = ''
        returntuple = self.collect_data()

        for k, v in returntuple[2].items():
            # Simple format once we implement  warn;crit;min;max
            # 'label'=value[UOM];[warn];[crit];[min];[max] 
            perfdata += '{}={};;;; '.format(k, v)
        
        return perfdata



# http://python-packaging.readthedocs.io/en/latest/minimal.html
# Help from: http://www.scotttorborg.com/python-packaging/minimal.html
# https://pypi.python.org/pypi?%3Aaction=list_classifiers
import os
from os import path
from setuptools import setup, find_packages
import rstcheck


mypackage_root_dir = 'webapp_health_check'

# function to check a readme file
def check_readme(file='README.rst'):
    """
    Checks readme rst file, to ensure it will upload to pypi and be formatted
    correctly.
    :param file:
    :return:
    """
    
    # Get the long description from the relevant file
    with open(file, encoding='utf-8') as f_object:
        readme_content = f_object.read()

    errors = list(rstcheck.check(readme_content))
    if errors:
        msg = 'There_path are errors in {}, errors \n {}'.format(file,
                                                            errors[0].message)
        raise SystemExit(msg)
    else:
        msg = 'No errors in {}'.format(file)
        print(msg)

# Get requirements for this package
here_path = path.abspath(path.dirname(__file__))
#with open(os.path.join(here_path, 'requirements.txt')) as f:
#    requires = [x.strip() for x in f if x.strip()]

install_requires=[
    'rstcheck',
    'requests>=2.19.1'
]

# Get the version from VERSION file
with open(os.path.join(mypackage_root_dir, 'VERSION')) as version_file:
    version = version_file.read().strip()

readme_path = path.join(here_path, 'README.rst')
# Get the long description from the relevant file
with open(readme_path, encoding='utf-8') as f:
    long_description = f.read()

# Check the readme
# when checking rst file you ensure it will be healthy to publish on pypi.org
check_readme(readme_path)


# Define setuptools specifications
setup(name='webapp_health_check',
      version=version,
      description='wen app health plugin',
      long_description=long_description,  # this is the file README.rst
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3 :: Only',
          'Topic :: System :: Monitoring',
      ],
      url='https://github.com/matiasgrana/webapp_health_check',
      author='Matias Graña',
      author_email='maicolmatiasg@gmail.com',
      license='MIT', # Choose your license
      # http://setuptools.readthedocs.io/en/latest/setuptools.html#using-find-packages
      packages=find_packages(),
      include_package_data=True,
      # use package_data if you want to include more files in the package
      #package_data={
      #    'data': 'src/data/*',  
      #},
      data_files=[('VERSION', ['{}/VERSION'.format(mypackage_root_dir)])],
      entry_points={
          'console_scripts': [
              # http://python-packaging.readthedocs.io/en/latest/command-line-scripts.html
              'webapp_health_check = webapp_health_check.__main__:main'
          ]
      },
      #install_requires=requires, # we have already readed requirements.txt in line 30
      # Use test_require  to add pytest requirements when using unit tests
      #tests_require=['pytest',
      #               'pytest-cov'],
      zip_safe=False)

